/*
 * Local Matters, Inc. Confidential
 * OCO Source Materials
 * (C) Local Matters, Inc. 2007. All Rights Reserved.
 */
package com.localmatters.flexiq.collector.producer;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import javax.ejb.Stateless;
import javax.jms.BytesMessage;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;
import javax.naming.InitialContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * The collector message producer implementation.
 *
 * @author Daniel Cavalcanti
 */
@Stateless()
@WebService()
public class ProducerWS {
    
    // **********************************************************************
    // Instance variables
    // **********************************************************************
    
    // Logging facility *****************************************************
    
    /**
     * The class logger.
     */
    private Log log = LogFactory.getLog(getClass());
    
    // Pre-configured JMS managed objects ***********************************
    
    /**
     * The JMS connection factory.
     */
    @Resource(mappedName="jms/CollectorConnectionFactory")
    private ConnectionFactory connectionFactory;
    
    /**
     * The JMS collector initialDestination.
     */
    @Resource(mappedName="jms/InitialDestination")
    private Destination initialDestination;
    
    // JMS Connection parameters ********************************************
    
    /**
     * The JMS connection username.
     */
    @Resource(name="username")
    private String username;
    
    /**
     * The JMS connection password.
     */
    @Resource(name="password")
    private String password;
    
    // JMS connectivity objects *********************************************
    
    /**
     * The JMS connection.
     */
    private Connection connection;
    
    /**
     * The JMS session.
     */
    private Session session;
    
    /**
     * The JMS message producer.
     */
    private MessageProducer producer;
    
    // **********************************************************************
    // Constructors
    // **********************************************************************
    
    /**
     * Creates a new instance of ProducerWS.
     */
    public ProducerWS() {
        if (log.isTraceEnabled()) log.trace("Creating collector producer bean.");
    }
    
    // **********************************************************************
    // Life-cycle methods
    // **********************************************************************
    
    /**
     * Initialize resources.
     */
    @PostConstruct()
    private void init() {
        
        // ******************************************************************
        // Log initial parameters
        // ******************************************************************
        
        if (log.isTraceEnabled()) log.trace("Initializing collector producer bean.");
        try {
            
            // **************************************************************
            // Establish JMS connectivity
            // **************************************************************
            
            if (connectionFactory == null) {
                if (log.isTraceEnabled()) log.trace("Connection factory not set. Retrieving from server context.");
                InitialContext ic = new InitialContext();
                connectionFactory = (ConnectionFactory) ic.lookup("jms/CollectorConnectionFactory");
            }
            
            if (log.isTraceEnabled()) log.trace("Establishing connection to JMS broker.");
            connection = connectionFactory.createConnection(username, password);

        } catch (JMSException ex) {
            log.error("Failed to establish JMS connectivity.", ex);
        } catch (Throwable ex) {
            log.error("Unexpected error.", ex);
        }
        
    }
    
    /**
     * Initializes the JMS connectivity objects: connection, session, and
     * producer.
     *
     * @throws JMSException If there is an exception/error.
     */
    private void initJMSConnectivity()
    throws JMSException {
        
        if (log.isTraceEnabled()) log.trace("Creating transacted session with JMS broker.");
        session = connection.createSession(true, Session.SESSION_TRANSACTED);
        
        if (log.isTraceEnabled()) log.trace("Creating message producer.");
        producer = session.createProducer(initialDestination);
        
        if (log.isTraceEnabled()) log.trace("Starting connection.");
        connection.start();
        
        if (log.isTraceEnabled()) log.trace("Collector producer bean started successfully.");
        
    }
    
    /**
     * Releases resources.
     */
    @PreDestroy()
    private void release() {
        
        // ******************************************************************
        // Release connectivity
        // ******************************************************************
        
        try {
            if (log.isTraceEnabled()) log.trace("Releasing JMS connectivity.");
            if (connection != null) connection.close();
            if (log.isTraceEnabled()) log.trace("Collector producer bean JMS connectivity resources released.");
        } catch (JMSException ex) {
            log.error("Failed to release JMS connectivity.", ex);
        } catch (Throwable ex) {
            log.error("Unexpected error.", ex);
        }
        
    }
    
    // **********************************************************************
    // ProducerService methods
    // **********************************************************************
    
    /**
     * Sends a set of collectorMessages into the system.
     *
     * @param collectorMessages The list of collector messages to send into the system.
     * @param username The queue username.
     * @param password The queue password.
     * @throws com.localmatters.flexiq.collector.producer.incoming.ProducerException If there is an exception/error.
     */
    @WebMethod()
    public void send(@WebParam(name="msgs") byte[][] collectorMessages, @WebParam(name="username") String username, @WebParam(name="password") String password)
    throws ProducerException {
        
        // ******************************************************************
        // Validate parameters
        // ******************************************************************
        
        if (log.isTraceEnabled()) log.trace("Validating web-service parameters.");
        
        if (username == null)
            throw new ProducerException(new NullPointerException("Username cannot be null."));
        if (password == null)
            throw new ProducerException(new NullPointerException("Password cannot be null."));
        if (!username.equals(this.username) || !password.equals(this.password))
            throw new ProducerException(new IllegalArgumentException("Invalid user credentials."));
        if (collectorMessages == null)
            throw new ProducerException(new NullPointerException("Messages cannot be null."));
        for (byte [] message : collectorMessages)
            if (message == null)
                throw new ProducerException(new NullPointerException("Message cannot be null."));
        
        // ******************************************************************
        // Send collectorMessages
        // ******************************************************************
        
        try {
            
            if (log.isTraceEnabled()) log.trace("Initializing JMS connectivity.");
            initJMSConnectivity();
            
            if (log.isTraceEnabled()) log.trace("Creating re-usable objects: stream-source and jaxb-unmarshaller.");
            
            if (log.isTraceEnabled()) log.trace("Processing messages: " + collectorMessages.length);
            for (byte[] collectorMessage : collectorMessages) {
                
                if (log.isTraceEnabled()) log.trace("Processing message:\n" + new String(collectorMessage));
                
                // **************************************************************
                // Create JMS message
                // **************************************************************
                
                if (log.isTraceEnabled()) log.trace("Creating JMS message.");
                BytesMessage jmsMessage = session.createBytesMessage();
                
                // **************************************************************
                // Conclude
                // **************************************************************
                
                if (log.isTraceEnabled()) log.trace("Writing collector message byte stream to JMS message.");
                jmsMessage.writeBytes(collectorMessage);
                
                if (log.isTraceEnabled()) log.trace("Sending JMS message.");
                producer.send(jmsMessage);
                if (log.isTraceEnabled()) log.trace("JMS message sent.");
                
            }
            
            session.close();
            
        } catch (Throwable ex) {
            log.error("Unexpected error.", ex);
            throw new ProducerException("Unexpected error.", ex);
        }
        
    }
    
    // **********************************************************************
    // Auxiliary methods
    // **********************************************************************
    
}
